@@ -463,7 +463,35 @@ graphene_matrix_is_identity (const graphene_matrix_t *m)
 bool
 graphene_matrix_is_2d (const graphene_matrix_t *m)
 {
+#if 0
+  float res[4];
+
+  graphene_simd4f_dup_4f (m->value.x, res);
+  if (!(graphene_fuzzy_equals (res[2], 0.f, 0.000001) &&
+        graphene_fuzzy_equals (res[3], 0.f, 0.000001)))
+    return false;
+
+  graphene_simd4f_dup_4f (m->value.y, res);
+  if (!(graphene_fuzzy_equals (res[2], 0.f, 0.000001) &&
+        graphene_fuzzy_equals (res[3], 0.f, 0.000001)))
+    return false;
+
+  graphene_simd4f_dup_4f (m->value.z, res);
+  if (!(graphene_fuzzy_equals (res[0], 0.f, 0.000001) &&
+        graphene_fuzzy_equals (res[1], 0.f, 0.000001) &&
+        graphene_fuzzy_equals (res[2], 1.f, 0.000001) &&
+        graphene_fuzzy_equals (res[3], 0.f, 0.000001)))
+    return false;
+
+  graphene_simd4f_dup_4f (m->value.w, res);
+  if (!(graphene_fuzzy_equals (res[2], 0.f, 0.000001) &&
+        graphene_fuzzy_equals (res[3], 1.f, 0.000001)))
+    return false;
+
+  return true;
+#else
   return graphene_simd4x4f_is_2d (&m->value);
+#endif
 }
 
 /**
@@ -541,7 +569,7 @@ graphene_matrix_init_from_2d (graphene_matrix_t *m,
                               double             y_0)
 {
   m->value = graphene_simd4x4f_init (graphene_simd4f_init ( xx,  yx, 0.f, 0.f),
-                                     graphene_simd4f_init ( yx,  yy, 0.f, 0.f),
+                                     graphene_simd4f_init ( xy,  yy, 0.f, 0.f),
                                      graphene_simd4f_init (0.f, 0.f, 1.f, 0.f),
                                      graphene_simd4f_init (x_0, y_0, 0.f, 1.f));
 
@@ -586,21 +614,37 @@ graphene_matrix_to_2d (const graphene_matrix_t *m,
                        double                  *x_0,
                        double                  *y_0)
 {
-  if (!graphene_simd4x4f_is_2d (&m->value))
+  float res[4];
+
+  graphene_simd4f_dup_4f (m->value.x, res);
+  if (res[2] != 0.f && res[3] != 0.f)
     return false;
 
   if (xx != NULL)
-    *xx = graphene_matrix_get_value (m, 0, 0);
+    *xx = res[0];
   if (yx != NULL)
-    *yx = graphene_matrix_get_value (m, 1, 0);
+    *yx = res[1];
+
+  graphene_simd4f_dup_4f (m->value.y, res);
+  if (res[2] != 0.f && res[3] != 0.f)
+    return false;
+
   if (xy != NULL)
-    *xy = graphene_matrix_get_value (m, 0, 1);
+    *xy = res[0];
   if (yy != NULL)
-    *yy = graphene_matrix_get_value (m, 1, 1);
+    *yy = res[1];
+
+  if (graphene_simd4f_cmp_neq (m->value.z, graphene_simd4f_init (0.f, 0., 1.f, 0.f)))
+    return false;
+
+  graphene_simd4f_dup_4f (m->value.w, res);
+  if (res[2] != 0.f && res[3] != 1.f)
+    return false;
+
   if (x_0 != NULL)
-    *x_0 = graphene_matrix_get_value (m, 3, 0);
+    *x_0 = res[0];
   if (y_0 != NULL)
-    *y_0 = graphene_matrix_get_value (m, 3, 1);
+    *y_0 = res[1];
 
   return true;
 }
